#
#    peepdf is a tool to analyse and modify PDF files
#    http://peepdf.eternal-todo.com
#    By Jose Miguel Esparza <jesparza AT eternal-todo.com>
#    Updated for Python 3 by Corey Forman (digitalsleuth - https://github.com/digitalsleuth/peepdf-3)
#    Copyright (C) 2012-2014 Jose Miguel Esparza
#
#    This file is part of peepdf.
#
#        peepdf is free software: you can redistribute it and/or modify
#        it under the terms of the GNU General Public License as published by
#        the Free Software Foundation, either version 3 of the License, or
#        (at your option) any later version.
#
#        peepdf is distributed in the hope that it will be useful,
#        but WITHOUT ANY WARRANTY; without even the implied warranty of
#        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
#        GNU General Public License for more details.
#
#        You should have received a copy of the GNU General Public License
#        along with peepdf.    If not, see <http://www.gnu.org/licenses/>.
#

"""
Created from the demonstration of the pythonaes package.

Copyright (c) 2010, Adam Newman http://www.caller9.com/
Licensed under the MIT license http://www.opensource.org/licenses/mit-license.php
"""

import sys
from aespython import key_expander, aes_cipher, cbc_mode


def decryptData(data, password=None, keyLength=None, mode="CBC"):
    """
    Method added for peepdf
    """
    decryptedData = ""
    if keyLength == None:
        keyLength = len(password) * 8
    if keyLength not in [128, 192, 256]:
        return (-1, "Bad length key in AES decryption process")
    try:
        iv = list(map(ord, data[:16]))
    except:
        iv = list(data[:16])
    try:
        key = list(map(ord, password))
    except:
        key = list(password)
    data = data[16:]
    if len(data) % 16 != 0:
        data = data[: -(len(data) % 16)]
    keyExpander = key_expander.KeyExpander(keyLength)
    expandedKey = keyExpander.expand(key)
    aesCipher = aes_cipher.AESCipher(expandedKey)
    if mode == "CBC":
        aesMode = cbc_mode.CBCMode(aesCipher, 16)
    aesMode.set_iv(iv)
    for i in range(0, len(data), 16):
        try:
            ciphertext = list(map(ord, data[i : i + 16]))
        except:
            ciphertext = list(data[i : i + 16])
        decryptedBytes = aesMode.decrypt_block(ciphertext)
        for byte in decryptedBytes:
            decryptedData += chr(byte)
    return (0, decryptedData)



#
#    peepdf is a tool to analyse and modify PDF files
#    http://peepdf.eternal-todo.com
#    By Jose Miguel Esparza <jesparza AT eternal-todo.com>
#    Updated for Python 3 by Corey Forman (digitalsleuth - https://github.com/digitalsleuth/peepdf-3)
#    Copyright (C) 2011-2017 Jose Miguel Esparza
#
#    This file is part of peepdf.
#
#        peepdf is free software: you can redistribute it and/or modify
#        it under the terms of the GNU General Public License as published by
#        the Free Software Foundation, either version 3 of the License, or
#        (at your option) any later version.
#
#        peepdf is distributed in the hope that it will be useful,
#        but WITHOUT ANY WARRANTY; without even the implied warranty of
#        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
#        GNU General Public License for more details.
#
#        You should have received a copy of the GNU General Public License
#        along with peepdf. If not, see <http://www.gnu.org/licenses/>.
#

"""
    Module with some misc functions
"""

import os, re, html.entities, json, requests


def clearScreen():
    """
    Simple method to clear the screen depending on the OS
    """
    if os.name == "nt":
        os.system("cls")
    elif os.name == "posix":
        os.system("reset")
    elif os.name == "mac":
        os.system("clear")


def countArrayElements(array):
    """
    Simple method to count the repetitions of elements in an array

            @param array: An array of elements
            @return: A tuple (elements,counters), where elements is a list with the distinct elements and counters is the list with the number of times they appear in the array
    """
    elements = []
    counters = []
    for element in array:
        if element in elements:
            indx = elements.index(element)
            counters[indx] += 1
        else:
            elements.append(element)
            counters.append(1)
    return elements, counters


def countNonPrintableChars(string):
    """
    Simple method to return the non printable characters found in an string

            @param string: A string
            @return: Number of non printable characters in the string
    """
    counter = 0
    for i in range(len(string)):
        if ord(string[i]) <= 31 or ord(string[i]) >= 127:
            counter += 1
    return counter


def decodeName(name):
    """
    Decode the given PDF name

            @param name: A PDFName string to decode
            @return: A tuple (status,statusContent), where statusContent is the decoded PDF name in case status = 0 or an error in case status = -1
    """
    decodedName = name
    hexNumbers = re.findall("#([0-9a-f]{2})", name, re.DOTALL | re.IGNORECASE)
    for hexNumber in hexNumbers:
        try:
            decodedName = decodedName.replace("#" + hexNumber, chr(int(hexNumber, 16)))
        except:
            return (-1, "Error decoding name")
    return (0, decodedName)


def decodeString(string):
    """
    Decode the given PDF string

            @param string: A PDFString to decode
            @return A tuple (status,statusContent), where statusContent is the decoded PDF string in case status = 0 or an error in case status = -1
    """
    decodedString = string
    octalNumbers = re.findall("\\\\([0-7]{1-3})", decodedString, re.DOTALL)
    for octal in octalNumbers:
        try:
            decodedString = decodedString.replace("\\\\" + octal, chr(int(octal, 8)))
        except:
            return (-1, "Error decoding string")
    return (0, decodedString)


def encodeName(name):
    """
    Encode the given PDF name

            @param name: A PDFName string to encode
            @return: A tuple (status,statusContent), where statusContent is the encoded PDF name in case status = 0 or an error in case status = -1
    """
    encodedName = ""
    if name[0] == "/":
        name = name[1:]
    for char in name:
        if char == "\0":
            encodedName += char
        else:
            try:
                hex = "%x" % ord(char)
                encodedName += "#" + hex
            except:
                return (-1, "Error encoding name")
    return (0, "/" + encodedName)


def encodeString(string):
    """
    Encode the given PDF string

            @param string: A PDFString to encode
            @return: A tuple (status,statusContent), where statusContent is the encoded PDF string in case status = 0 or an error in case status = -1
    """
    encodedString = ""
    try:
        for char in string:
            octal = "%o" % ord(char)
            encodedString += "\\" + (3 - len(octal)) * "0" + octal
    except:
        return (-1, "Error encoding string")
    return (0, encodedString)


def escapeRegExpString(string):
    """
    Escape the given string to include it as a regular expression

    @param string: A regular expression to be escaped
    @return: Escaped string
    """
    toEscapeChars = ["\\", "(", ")", ".", "|", "^", "$", "*", "+", "?", "[", "]"]
    escapedValue = ""
    for i in range(len(string)):
        if string[i] in toEscapeChars:
            escapedValue += f"\\{string[i]}"
        else:
            escapedValue += string[i]
    return escapedValue


def escapeString(string):
    """
    Escape the given string

            @param string: A string to be escaped
            @return: Escaped string
    """
    toEscapeChars = ["\\", "(", ")"]
    escapedValue = ""
    for i in range(len(string)):
        if string[i] in toEscapeChars and (i == 0 or string[i - 1] != "\\"):
            if string[i] == "\\":
                if len(string) > i + 1 and re.match("[0-7]", string[i + 1]):
                    escapedValue += string[i]
                else:
                    escapedValue += "\\" + string[i]
            else:
                escapedValue += "\\" + string[i]
        elif string[i] == "\r":
            escapedValue += "\\r"
        elif string[i] == "\n":
            escapedValue += "\\n"
        elif string[i] == "\t":
            escapedValue += "\\t"
        elif string[i] == "\b":
            escapedValue += "\\b"
        elif string[i] == "\f":
            escapedValue += "\\f"
        else:
            escapedValue += string[i]
    return escapedValue


def getBitsFromNum(num, bitsPerComponent=8):
    """
    Makes the conversion between number and bits

    @param num: Number to be converted
    @param bitsPerComponent: Number of bits needed to represent a component
    @return: A tuple (status,statusContent), where statusContent is the string containing the resulting bits in case status = 0 or an error in case status = -1
    """
    if not isinstance(num, int):
        return (-1, "num must be an integer")
    if not isinstance(bitsPerComponent, int):
        return (-1, "bitsPerComponent must be an integer")
    try:
        bitsRepresentation = bin(num)
        bitsRepresentation = bitsRepresentation.replace("0b", "")
        mod = len(bitsRepresentation) % 8
        if mod != 0:
            bitsRepresentation = "0" * (8 - mod) + bitsRepresentation
        bitsRepresentation = bitsRepresentation[-1 * bitsPerComponent :]
    except:
        return (-1, "Error in conversion from number to bits")
    return (0, bitsRepresentation)


def getNumsFromBytes(bytes, bitsPerComponent=8):
    """
    Makes the conversion between bytes and numbers, depending on the number of bits used per component.

    @param bytes: String representing the bytes to be converted
    @param bitsPerComponent: Number of bits needed to represent a component
    @return: A tuple (status,statusContent), where statusContent is a list of numbers in case status = 0 or an error in case status = -1
    """
    if not isinstance(bytes, str):
        return (-1, "bytes must be a string")
    if not isinstance(bitsPerComponent, int):
        return (-1, "bitsPerComponent must be an integer")
    outputComponents = []
    bitsStream = ""
    for byte in bytes:
        try:
            bitsRepresentation = bin(ord(byte))
            bitsRepresentation = bitsRepresentation.replace("0b", "")
            bitsRepresentation = (
                "0" * (8 - len(bitsRepresentation)) + bitsRepresentation
            )
            bitsStream += bitsRepresentation
        except:
            return (-1, "Error in conversion from bytes to bits")

    try:
        for i in range(0, len(bitsStream), bitsPerComponent):
            bytes = ""
            bits = bitsStream[i : i + bitsPerComponent]
            num = int(bits, 2)
            outputComponents.append(num)
    except:
        return (-1, "Error in conversion from bits to bytes")
    return (0, outputComponents)


def getBytesFromBits(bitsStream):
    """
    Makes the conversion between bits and bytes.

    @param bitsStream: String representing a chain of bits
    @return: A tuple (status,statusContent), where statusContent is the string containing the resulting bytes in case status = 0 or an error in case status = -1
    """
    if not isinstance(bitsStream, str):
        return (-1, "The bitsStream must be a string")
    bytes = ""
    if re.match("[01]*$", bitsStream):
        try:
            for i in range(0, len(bitsStream), 8):
                bits = bitsStream[i : i + 8]
                byte = chr(int(bits, 2))
                bytes += byte
        except:
            return (-1, "Error in conversion from bits to bytes")
        return (0, bytes)
    else:
        return (-1, "The format of the bit stream is not correct")


def getBytesFromFile(filename, offset, numBytes):
    """
    Returns the number of bytes specified from a file, starting from the offset specified

            @param filename: Name of the file
            @param offset: Bytes offset
            @param numBytes: Number of bytes to retrieve
            @return: A tuple (status,statusContent), where statusContent is the bytes read in case status = 0 or an error in case status = -1
    """
    if not isinstance(offset, int) or not isinstance(numBytes, int):
        return (-1, "The offset and the number of bytes must be integers")
    if os.path.exists(filename):
        fileSize = os.path.getsize(filename)
        bytesFile = open(filename, "rb")
        bytesFile.seek(offset)
        if offset + numBytes > fileSize:
            byteVal = bytesFile.read()
        else:
            byteVal = bytesFile.read(numBytes)
        bytesFile.close()
        return (0, byteVal)
    else:
        return (-1, "File does not exist")


def hexToString(hexString):
    """
    Simple method to convert an hexadecimal string to ascii string

    @param hexString: A string in hexadecimal format
    @return: A tuple (status,statusContent), where statusContent is an ascii string in case status = 0 or an error in case status = -1
    """
    string = ""
    if len(hexString) % 2 != 0:
        hexString = "0" + hexString
    try:
        for i in range(0, len(hexString), 2):
            string += chr(int(hexString[i] + hexString[i + 1], 16))
    except:
        return (-1, "Error in hexadecimal conversion")
    return (0, string)


def numToHex(num, numBytes):
    """
    Given a number returns its hexadecimal format with the specified length, adding '\0' if necessary

            @param num: A number (int)
            @param numBytes: Length of the output (int)
            @return: A tuple (status,statusContent), where statusContent is a number in hexadecimal format in case status = 0 or an error in case status = -1
    """
    hexString = ""
    if not isinstance(num, int):
        return (-1, "Bad number")
    try:
        hexNumber = hex(num)[2:]
        if len(hexNumber) % 2 != 0:
            hexNumber = "0" + hexNumber
        for i in range(0, len(hexNumber) - 1, 2):
            hexString += chr(int(hexNumber[i] + hexNumber[i + 1], 16))
        hexString = "\0" * (numBytes - len(hexString)) + hexString
    except:
        return (-1, "Error in hexadecimal conversion")
    return (0, hexString)


def numToString(num, numDigits):
    """
    Given a number returns its string format with the specified length, adding '0' if necessary

            @param num: A number (int)
            @param numDigits: Length of the output string (int)
            @return: A tuple (status,statusContent), where statusContent is a number in string format in case status = 0 or an error in case status = -1
    """
    if not isinstance(num, int):
        return (-1, "Bad number")
    strNum = str(num)
    if numDigits < len(strNum):
        return (-1, "Bad digit number")
    for i in range(numDigits - len(strNum)):
        strNum = "0" + strNum
    return (0, strNum)


def unescapeHTMLEntities(text):
    """
    Removes HTML or XML character references and entities from a text string.

    @param text The HTML (or XML) source text.
    @return The plain text, as a Unicode string, if necessary.

    Author: Fredrik Lundh
    Source: http://effbot.org/zone/re-sub.htm#unescape-html
    """

    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return chr(int(text[3:-1], 16))
                else:
                    return chr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = chr(html.entities.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text  # leave as is

    return re.sub(r"&#?\w+;", fixup, text)


def unescapeString(string):
    """
    Unescape the given string

            @param string: An escaped string
            @return: Unescaped string
    """
    toUnescapeChars = ["\\", "(", ")"]
    unescapedValue = ""
    i = 0
    while i < len(string):
        if string[i] == "\\" and i != len(string) - 1:
            if string[i + 1] in toUnescapeChars:
                if string[i + 1] == "\\":
                    unescapedValue += "\\"
                    i += 1
                else:
                    pass
            elif string[i + 1] == "r":
                i += 1
                unescapedValue += "\r"
            elif string[i + 1] == "n":
                i += 1
                unescapedValue += "\n"
            elif string[i + 1] == "t":
                i += 1
                unescapedValue += "\t"
            elif string[i + 1] == "b":
                i += 1
                unescapedValue += "\b"
            elif string[i + 1] == "f":
                i += 1
                unescapedValue += "\f"
            else:
                unescapedValue += string[i]
        else:
            unescapedValue += string[i]
        i += 1
    return unescapedValue


def vtcheck(md5, vtKey):
    """
    Function to check a hash on VirusTotal and get the report summary

    @param md5: The MD5 to check (hexdigest)
    @param vtKey: The VirusTotal API key needed to perform the request
    @return: A dictionary with the result of the request
    """
    vtUrl = f"https://www.virustotal.com/api/v3/files/{md5}"
    headers = {"accept": "application/json", "x-apikey": vtKey}
    try:
        response = requests.get(vtUrl, headers=headers)
        jsonResponse = response.json()
    except:
        return (-1, f"The request to VirusTotal failed")
    try:
        jsonDict = jsonResponse
    except:
        return (
            -1,
            "An error has occurred while parsing the JSON response from VirusTotal",
        )
    return (0, jsonDict)



#
#    peepdf is a tool to analyse and modify PDF files
#    http://peepdf.eternal-todo.com
#    By Jose Miguel Esparza <jesparza AT eternal-todo.com>
#    Updated for Python 3 by Corey Forman (digitalsleuth - https://github.com/digitalsleuth/peepdf-3)
#    Copyright (C) 2014 Jose Miguel Esparza
#
#    This file is part of peepdf.
#
#        peepdf is free software: you can redistribute it and/or modify
#        it under the terms of the GNU General Public License as published by
#        the Free Software Foundation, either version 3 of the License, or
#        (at your option) any later version.
#
#        peepdf is distributed in the hope that it will be useful,
#        but WITHOUT ANY WARRANTY; without even the implied warranty of
#        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
#        GNU General Public License for more details.
#
#        You should have received a copy of the GNU General Public License
#        along with peepdf.    If not, see <http://www.gnu.org/licenses/>.
#

# Python version of the jjdecode function written by Syed Zainudeen
# http://csc.cs.utm.my/syed/images/files/jjdecode/jjdecode.html
# +NCR/CRC! [ReVeRsEr] - crackinglandia@gmail.com
#
# The original algorithm was written in Javascript by Yosuke Hasegawa (http://utf-8.jp/public/jjencode.html)
#
# Modified to integrate it with peepdf

import re, sys


class JJDecoder(object):
    def __init__(self, jj_encoded_data):
        self.encoded_str = jj_encoded_data

    def clean(self):
        self.encoded_str = re.sub("^\s+|\s+$", "", self.encoded_str)

    def checkPalindrome(self):
        startpos = -1
        endpos = -1
        gv, gvl = -1, -1

        index = self.encoded_str.find('"\'\\"+\'+",')

        if index == 0:
            startpos = self.encoded_str.find('$$+"\\""+') + 8
            endpos = self.encoded_str.find('"\\"")())()')
            gv = self.encoded_str[index + 9 : self.encoded_str.find("=~[]")]
            gvl = len(gv)
        else:
            gv = self.encoded_str[0 : self.encoded_str.find("=")]
            gvl = len(gv)
            startpos = self.encoded_str.find('"\\""+') + 5
            endpos = self.encoded_str.find('"\\"")())()')

        return (startpos, endpos, gv, gvl)

    def decode(self):
        self.clean()
        startpos, endpos, gv, gvl = self.checkPalindrome()

        if startpos == endpos:
            return (-1, "There is no data to decode")

        data = self.encoded_str[startpos:endpos]

        b = [
            "___+",
            "__$+",
            "_$_+",
            "_$$+",
            "$__+",
            "$_$+",
            "$$_+",
            "$$$+",
            "$___+",
            "$__$+",
            "$_$_+",
            "$_$$+",
            "$$__+",
            "$$_$+",
            "$$$_+",
            "$$$$+",
        ]

        str_l = '(![]+"")[' + gv + "._$_]+"
        str_o = gv + "._$+"
        str_t = gv + ".__+"
        str_u = gv + "._+"

        str_hex = gv + "."

        str_s = '"'
        gvsig = gv + "."

        str_quote = '\\\\\\"'
        str_slash = "\\\\\\\\"

        str_lower = '\\\\"+'
        str_upper = '\\\\"+' + gv + "._+"

        str_end = '"+'

        out = ""
        while data != "":
            # l o t u
            if data.find(str_l) == 0:
                data = data[len(str_l) :]
                out += "l"
                continue
            elif data.find(str_o) == 0:
                data = data[len(str_o) :]
                out += "o"
                continue
            elif data.find(str_t) == 0:
                data = data[len(str_t) :]
                out += "t"
                continue
            elif data.find(str_u) == 0:
                data = data[len(str_u) :]
                out += "u"
                continue

            # 0123456789abcdef
            if data.find(str_hex) == 0:
                data = data[len(str_hex) :]

                for i in range(len(b)):
                    if data.find(b[i]) == 0:
                        data = data[len(b[i]) :]
                        out += "%x" % i
                        break
                continue

            # start of s block
            if data.find(str_s) == 0:
                data = data[len(str_s) :]

                # check if "R
                if data.find(str_upper) == 0:  # r4 n >= 128
                    data = data[len(str_upper) :]  # skip sig
                    ch_str = ""
                    for i in range(2):  # shouldn't be more than 2 hex chars
                        # gv + "."+b[ c ]
                        if data.find(gvsig) == 0:
                            data = data[len(gvsig) :]
                            for k in range(len(b)):  # for every entry in b
                                if data.find(b[k]) == 0:
                                    data = data[len(b[k]) :]
                                    ch_str = "%x" % k
                                    break
                        else:
                            break

                    out += chr(int(ch_str, 16))
                    continue

                elif data.find(str_lower) == 0:  # r3 check if "R // n < 128
                    data = data[len(str_lower) :]  # skip sig

                    ch_str = ""
                    ch_lotux = ""
                    temp = ""
                    b_checkR1 = 0
                    for j in range(3):  # shouldn't be more than 3 octal chars
                        if j > 1:  # lotu check
                            if data.find(str_l) == 0:
                                data = data[len(str_l) :]
                                ch_lotux = "l"
                                break
                            elif data.find(str_o) == 0:
                                data = data[len(str_o) :]
                                ch_lotux = "o"
                                break
                            elif data.find(str_t) == 0:
                                data = data[len(str_t) :]
                                ch_lotux = "t"
                                break
                            elif data.find(str_u) == 0:
                                data = data[len(str_u) :]
                                ch_lotux = "u"
                                break

                        # gv + "."+b[ c ]
                        if data.find(gvsig) == 0:
                            temp = data[len(gvsig) :]
                            for k in range(8):  # for every entry in b octal
                                if temp.find(b[k]) == 0:
                                    if int(ch_str + str(k), 8) > 128:
                                        b_checkR1 = 1
                                        break

                                    ch_str += str(k)
                                    data = data[len(gvsig) :]  # skip gvsig
                                    data = data[len(b[k]) :]
                                    break

                            if b_checkR1 == 1:
                                if data.find(str_hex) == 0:  # 0123456789abcdef
                                    data = data[len(str_hex) :]
                                    # check every element of hex decode string for a match
                                    for i in range(len(b)):
                                        if data.find(b[i]) == 0:
                                            data = data[len(b[i]) :]
                                            ch_lotux = "%x" % i
                                            break
                                    break
                        else:
                            break

                    out += chr(int(ch_str, 8)) + ch_lotux
                    continue

                else:  # "S ----> "SR or "S+
                    # if there is, loop s until R 0r +
                    # if there is no matching s block, throw error

                    match = 0
                    n = None

                    # searching for matching pure s block
                    while True:
                        n = ord(data[0])
                        if data.find(str_quote) == 0:
                            data = data[len(str_quote) :]
                            out += '"'
                            match += 1
                            continue
                        elif data.find(str_slash) == 0:
                            data = data[len(str_slash) :]
                            out += "\\"
                            match += 1
                            continue
                        elif data.find(str_end) == 0:  # reached end off S block ? +
                            if match == 0:
                                return (-1, "+ No match S block")
                            data = data[len(str_end) :]
                            break  # step out of the while loop
                        elif (
                            data.find(str_upper) == 0
                        ):  # r4 reached end off S block ? - check if "R n >= 128z
                            if match == 0:
                                return (-1, "No match S block n>128")
                            data = data[len(str_upper) :]  # skip sig

                            ch_str = ""
                            ch_lotux = ""

                            for j in range(10):  # shouldn't be more than 10 hex chars
                                if j > 1:  # lotu check
                                    if data.find(str_l) == 0:
                                        data = data[len(str_l) :]
                                        ch_lotux = "l"
                                        break
                                    elif data.find(str_o) == 0:
                                        data = data[len(str_o) :]
                                        ch_lotux = "o"
                                        break
                                    elif data.find(str_t) == 0:
                                        data = data[len(str_t) :]
                                        ch_lotux = "t"
                                        break
                                    elif data.find(str_u) == 0:
                                        data = data[len(str_u) :]
                                        ch_lotux = "u"
                                        break

                                # gv + "."+b[ c ]
                                if data.find(gvsig) == 0:
                                    data = data[len(gvsig) :]  # skip gvsig
                                    for k in range(len(b)):  # for every entry in b
                                        if data.find(b[k]) == 0:
                                            data = data[len(b[k]) :]
                                            ch_str += "%x" % k
                                            break
                                else:
                                    break  # done
                            out += chr(int(ch_str, 16))
                            break  # step out of the while loop
                        elif data.find(str_lower) == 0:  # r3 check if "R // n < 128
                            if match == 0:
                                return (-1, "No match S block n<128!!")

                            data = data[len(str_lower) :]  # skip sig

                            ch_str = ""
                            ch_lotux = ""
                            temp = ""
                            b_checkR1 = 0

                            for j in range(3):  # shouldn't be more than 3 octal chars
                                if j > 1:  # lotu check
                                    if data.find(str_l) == 0:
                                        data = data[len(str_l) :]
                                        ch_lotux = "l"
                                        break
                                    elif data.find(str_o) == 0:
                                        data = data[len(str_o) :]
                                        ch_lotux = "o"
                                        break
                                    elif data.find(str_t) == 0:
                                        data = data[len(str_t) :]
                                        ch_lotux = "t"
                                        break
                                    elif data.find(str_u) == 0:
                                        data = data[len(str_u) :]
                                        ch_lotux = "u"
                                        break

                                # gv + "."+b[ c ]
                                if data.find(gvsig) == 0:
                                    temp = data[len(gvsig) :]
                                    for k in range(8):  # for every entry in b octal
                                        if temp.find(b[k]) == 0:
                                            if int(ch_str + str(k), 8) > 128:
                                                b_checkR1 = 1
                                                break

                                            ch_str += str(k)
                                            data = data[len(gvsig) :]  # skip gvsig
                                            data = data[len(b[k]) :]
                                            break

                                    if b_checkR1 == 1:
                                        if data.find(str_hex) == 0:  # 0123456789abcdef
                                            data = data[len(str_hex) :]
                                            # check every element of hex decode string for a match
                                            for i in range(len(b)):
                                                if data.find(b[i]) == 0:
                                                    data = data[len(b[i]) :]
                                                    ch_lotux = "%x" % i
                                                    break
                                else:
                                    break
                            out += chr(int(ch_str, 8)) + ch_lotux
                            break  # step out of the while loop
                        elif (
                            (0x21 <= n and n <= 0x2F)
                            or (0x3A <= n and n <= 0x40)
                            or (0x5B <= n and n <= 0x60)
                            or (0x7B <= n and n <= 0x7F)
                        ):
                            out += data[0]
                            data = data[1:]
                            match += 1
                    continue
            return (-1, "No match in the code!!")
            break
        return (0, out)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
ccitt.py

TODO
http://tools.ietf.org/pdf/rfc804.pdf
http://code.google.com/p/origami-pdf/source/browse/lib/origami/filters/ccitt.rb
"""
__author__ = "Binjo"
__version__ = "0.1"
__date__ = "2012-04-08 14:30:05"


class BitWriterException(Exception):
    pass


class BitWriter:
    """
    BitWriter Class
    """

    def __init__(
        self,
    ):
        """
        Init
        """
        self._data = ""
        self._last_byte = None
        self._bit_ptr = 0

    @property
    def data(self):
        """
        return self._data
        """
        return self._data

    def write(self, data, length):
        """
        write function
        """
        if not (length >= 0 and (1 << length) > data):
            raise BitWriterException("Invalid data length")

        if length == 8 and not self._last_byte and self._bit_ptr == 0:
            self._data += chr(data)
            return

        while length > 0:
            if length >= 8 - self._bit_ptr:
                length -= 8 - self._bit_ptr
                if not self._last_byte:
                    self._last_byte = 0
                self._last_byte |= (data >> length) & ((1 << (8 - self._bit_ptr)) - 1)

                data &= (1 << length) - 1
                self._data += chr(self._last_byte)
                self._last_byte = None
                self._bit_ptr = 0
            else:
                if not self._last_byte:
                    self._last_byte = 0
                self._last_byte |= (data & ((1 << length) - 1)) << (
                    8 - self._bit_ptr - length
                )
                self._bit_ptr += length

                if self._bit_ptr == 8:
                    self._data += chr(self._last_byte)
                    self._last_byte = None
                    self._bit_ptr = 0

                length = 0


class BitReaderException(Exception):
    pass


class BitReader:
    """ """

    def __init__(self, data):
        """ """
        self._data = data
        self._byte_ptr, self._bit_ptr = 0, 0

    def reset(self):
        """ """
        self._byte_ptr, self._bit_ptr = 0, 0

    @property
    def eod_p(self):
        """ """
        return self._byte_ptr >= len(self._data)

    @property
    def pos(self):
        """Comment"""
        return (self._byte_ptr << 3) + self._bit_ptr

    @property
    def size(self):
        """Comment"""
        return len(self._data) << 3

    @pos.setter
    def pos(self, bits):
        """Comment"""
        if bits > self.size:
            raise BitReaderException("Pointer position out of data")

        pbyte = bits >> 3
        pbit = bits - (pbyte << 3)
        self._byte_ptr, self._bit_ptr = pbyte, pbit

    def peek(self, length):
        """Comment"""
        if length <= 0:
            raise BitReaderException("Invalid read length")
        elif (self.pos + length) > self.size:
            raise BitReaderException("Insufficient data")

        n = 0
        byte_ptr, bit_ptr = self._byte_ptr, self._bit_ptr

        while length > 0:
            byte = ord(self._data[byte_ptr])

            if length > 8 - bit_ptr:
                length -= 8 - bit_ptr
                n |= (byte & ((1 << (8 - bit_ptr)) - 1)) << length

                byte_ptr += 1
                bit_ptr = 0
            else:
                n |= (byte >> (8 - bit_ptr - length)) & ((1 << length) - 1)
                length = 0

        return n

    def read(self, length):
        """Comment"""
        n = self.peek(length)
        self.pos += length

        return n


def codeword(bits):
    """return tuple rather than list, since list is not hashable..."""
    return (int(bits, 2), len(bits))


class CCITTFax:
    """Comment"""

    EOL = codeword("000000000001")
    RTC = codeword("000000000001" * 6)

    WHITE_TERMINAL_ENCODE_TABLE = {
        0: codeword("00110101"),
        1: codeword("000111"),
        2: codeword("0111"),
        3: codeword("1000"),
        4: codeword("1011"),
        5: codeword("1100"),
        6: codeword("1110"),
        7: codeword("1111"),
        8: codeword("10011"),
        9: codeword("10100"),
        10: codeword("00111"),
        11: codeword("01000"),
        12: codeword("001000"),
        13: codeword("000011"),
        14: codeword("110100"),
        15: codeword("110101"),
        16: codeword("101010"),
        17: codeword("101011"),
        18: codeword("0100111"),
        19: codeword("0001100"),
        20: codeword("0001000"),
        21: codeword("0010111"),
        22: codeword("0000011"),
        23: codeword("0000100"),
        24: codeword("0101000"),
        25: codeword("0101011"),
        26: codeword("0010011"),
        27: codeword("0100100"),
        28: codeword("0011000"),
        29: codeword("00000010"),
        30: codeword("00000011"),
        31: codeword("00011010"),
        32: codeword("00011011"),
        33: codeword("00010010"),
        34: codeword("00010011"),
        35: codeword("00010100"),
        36: codeword("00010101"),
        37: codeword("00010110"),
        38: codeword("00010111"),
        39: codeword("00101000"),
        40: codeword("00101001"),
        41: codeword("00101010"),
        42: codeword("00101011"),
        43: codeword("00101100"),
        44: codeword("00101101"),
        45: codeword("00000100"),
        46: codeword("00000101"),
        47: codeword("00001010"),
        48: codeword("00001011"),
        49: codeword("01010010"),
        50: codeword("01010011"),
        51: codeword("01010100"),
        52: codeword("01010101"),
        53: codeword("00100100"),
        54: codeword("00100101"),
        55: codeword("01011000"),
        56: codeword("01011001"),
        57: codeword("01011010"),
        58: codeword("01011011"),
        59: codeword("01001010"),
        60: codeword("01001011"),
        61: codeword("00110010"),
        62: codeword("00110011"),
        63: codeword("00110100"),
    }

    WHITE_TERMINAL_DECODE_TABLE = dict(
        (v, k) for k, v in WHITE_TERMINAL_ENCODE_TABLE.items()
    )

    BLACK_TERMINAL_ENCODE_TABLE = {
        0: codeword("0000110111"),
        1: codeword("010"),
        2: codeword("11"),
        3: codeword("10"),
        4: codeword("011"),
        5: codeword("0011"),
        6: codeword("0010"),
        7: codeword("00011"),
        8: codeword("000101"),
        9: codeword("000100"),
        10: codeword("0000100"),
        11: codeword("0000101"),
        12: codeword("0000111"),
        13: codeword("00000100"),
        14: codeword("00000111"),
        15: codeword("000011000"),
        16: codeword("0000010111"),
        17: codeword("0000011000"),
        18: codeword("0000001000"),
        19: codeword("00001100111"),
        20: codeword("00001101000"),
        21: codeword("00001101100"),
        22: codeword("00000110111"),
        23: codeword("00000101000"),
        24: codeword("00000010111"),
        25: codeword("00000011000"),
        26: codeword("000011001010"),
        27: codeword("000011001011"),
        28: codeword("000011001100"),
        29: codeword("000011001101"),
        30: codeword("000001101000"),
        31: codeword("000001101001"),
        32: codeword("000001101010"),
        33: codeword("000001101011"),
        34: codeword("000011010010"),
        35: codeword("000011010011"),
        36: codeword("000011010100"),
        37: codeword("000011010101"),
        38: codeword("000011010110"),
        39: codeword("000011010111"),
        40: codeword("000001101100"),
        41: codeword("000001101101"),
        42: codeword("000011011010"),
        43: codeword("000011011011"),
        44: codeword("000001010100"),
        45: codeword("000001010101"),
        46: codeword("000001010110"),
        47: codeword("000001010111"),
        48: codeword("000001100100"),
        49: codeword("000001100101"),
        50: codeword("000001010010"),
        51: codeword("000001010011"),
        52: codeword("000000100100"),
        53: codeword("000000110111"),
        54: codeword("000000111000"),
        55: codeword("000000100111"),
        56: codeword("000000101000"),
        57: codeword("000001011000"),
        58: codeword("000001011001"),
        59: codeword("000000101011"),
        60: codeword("000000101100"),
        61: codeword("000001011010"),
        62: codeword("000001100110"),
        63: codeword("000001100111"),
    }

    BLACK_TERMINAL_DECODE_TABLE = dict(
        (v, k) for k, v in BLACK_TERMINAL_ENCODE_TABLE.items()
    )

    WHITE_CONFIGURATION_ENCODE_TABLE = {
        64: codeword("11011"),
        128: codeword("10010"),
        192: codeword("010111"),
        256: codeword("0110111"),
        320: codeword("00110110"),
        384: codeword("00110111"),
        448: codeword("01100100"),
        512: codeword("01100101"),
        576: codeword("01101000"),
        640: codeword("01100111"),
        704: codeword("011001100"),
        768: codeword("011001101"),
        832: codeword("011010010"),
        896: codeword("011010011"),
        960: codeword("011010100"),
        1024: codeword("011010101"),
        1088: codeword("011010110"),
        1152: codeword("011010111"),
        1216: codeword("011011000"),
        1280: codeword("011011001"),
        1344: codeword("011011010"),
        1408: codeword("011011011"),
        1472: codeword("010011000"),
        1536: codeword("010011001"),
        1600: codeword("010011010"),
        1664: codeword("011000"),
        1728: codeword("010011011"),
        1792: codeword("00000001000"),
        1856: codeword("00000001100"),
        1920: codeword("00000001001"),
        1984: codeword("000000010010"),
        2048: codeword("000000010011"),
        2112: codeword("000000010100"),
        2176: codeword("000000010101"),
        2240: codeword("000000010110"),
        2340: codeword("000000010111"),
        2368: codeword("000000011100"),
        2432: codeword("000000011101"),
        2496: codeword("000000011110"),
        2560: codeword("000000011111"),
    }

    WHITE_CONFIGURATION_DECODE_TABLE = dict(
        (v, k) for k, v in WHITE_CONFIGURATION_ENCODE_TABLE.items()
    )

    BLACK_CONFIGURATION_ENCODE_TABLE = {
        64: codeword("0000001111"),
        128: codeword("000011001000"),
        192: codeword("000011001001"),
        256: codeword("000001011011"),
        320: codeword("000000110011"),
        384: codeword("000000110100"),
        448: codeword("000000110101"),
        512: codeword("0000001101100"),
        576: codeword("0000001101101"),
        640: codeword("0000001001010"),
        704: codeword("0000001001011"),
        768: codeword("0000001001100"),
        832: codeword("0000001001101"),
        896: codeword("0000001110010"),
        960: codeword("0000001110011"),
        1024: codeword("0000001110100"),
        1088: codeword("0000001110101"),
        1152: codeword("0000001110110"),
        1216: codeword("0000001110111"),
        1280: codeword("0000001010010"),
        1344: codeword("0000001010011"),
        1408: codeword("0000001010100"),
        1472: codeword("0000001010101"),
        1536: codeword("0000001011010"),
        1600: codeword("0000001011011"),
        1664: codeword("0000001100100"),
        1728: codeword("0000001100101"),
        1792: codeword("00000001000"),
        1856: codeword("00000001100"),
        1920: codeword("00000001001"),
        1984: codeword("000000010010"),
        2048: codeword("000000010011"),
        2112: codeword("000000010100"),
        2176: codeword("000000010101"),
        2240: codeword("000000010110"),
        2340: codeword("000000010111"),
        2368: codeword("000000011100"),
        2432: codeword("000000011101"),
        2496: codeword("000000011110"),
        2560: codeword("000000011111"),
    }

    BLACK_CONFIGURATION_DECODE_TABLE = dict(
        (v, k) for k, v in BLACK_CONFIGURATION_ENCODE_TABLE.items()
    )

    def __init__(
        self,
    ):
        """Comment"""
        self._decoded = []

    def decode(
        self,
        stream,
        k=0,
        eol=False,
        byteAlign=False,
        columns=1728,
        rows=0,
        eob=True,
        blackIs1=False,
        damagedRowsBeforeError=0,
    ):
        """Comment"""
        # FIXME seems not stick to the spec? default is false, but if not set as true, it won't decode 6cc2a162e08836f7d50d461a9fc136fe correctly
        byteAlign = True

        if blackIs1:
            white, black = 0, 1
        else:
            white, black = 1, 0

        bitr = BitReader(stream)
        bitw = BitWriter()

        while not (bitr.eod_p or rows == 0):
            current_color = white
            if byteAlign and bitr.pos % 8 != 0:
                bitr.pos += 8 - (bitr.pos % 8)

            if eob and bitr.peek(self.RTC[1]) == self.RTC[0]:
                bitr.pos += self.RTC[1]
                break

            if bitr.peek(self.EOL[1]) != self.EOL[0]:
                if eol:
                    raise Exception(
                        f"No end-of-line pattern found (at bit pos {bitr.pos}/{bitr.size})"
                    )
            else:
                bitr.pos += self.EOL[1]

            line_length = 0
            while line_length < columns:
                if current_color == white:
                    bit_length = self.get_white_bits(bitr)
                else:
                    bit_length = self.get_black_bits(bitr)
                if bit_length == None:
                    raise Exception(
                        f"Unfinished line (at bit pos {bitr.pos}/{bitr.size}), {bitw.data}"
                    )

                line_length += bit_length
                if line_length > columns:
                    raise Exception(
                        f"Line is too long (at bit pos {bitr.pos}/{bitr.size})"
                    )

                bitw.write((current_color << bit_length) - current_color, bit_length)

                current_color ^= 1

            rows -= 1
        return bitw.data

    def get_white_bits(self, bitr):
        """Comment"""
        return self.get_color_bits(
            bitr,
            self.WHITE_CONFIGURATION_DECODE_TABLE,
            self.WHITE_TERMINAL_DECODE_TABLE,
        )

    def get_black_bits(self, bitr):
        """Comment"""
        return self.get_color_bits(
            bitr,
            self.BLACK_CONFIGURATION_DECODE_TABLE,
            self.BLACK_TERMINAL_DECODE_TABLE,
        )

    def get_color_bits(self, bitr, config_words, term_words):
        """Comment"""
        bits = 0
        check_conf = True

        while check_conf:
            check_conf = False

            for i in range(2, 14):
                codeword = bitr.peek(i)
                config_value = config_words.get((codeword, i), None)

                if config_value is not None:
                    bitr.pos += i
                    bits += config_value
                    if config_value == 2560:
                        check_conf = True
                    break

            for i in range(2, 14):
                codeword = bitr.peek(i)
                term_value = term_words.get((codeword, i), None)

                if term_value is not None:
                    bitr.pos += i
                    bits += term_value

                    return bits

        return None



from setuptools import setup

setup(
    name="peepdf-3",
    version="2.2.0",
    author="Jose Miguel Esparza, Corey Forman",
    license="GNU GPLv3",
    url="https://github.com/digitalsleuth/peepdf-3",
    description=("The original peepdf, ported to Python 3, and packaged in a setup"),
    install_requires=[
        "requests",
        "pypdf",
        "jsbeautifier",
        "colorama",
        "Pillow",
        "pythonaes",
        "pylibemu",
        "lxml",
    ],
    entry_points={
        "console_scripts": [
            "peepdf = peepdf.main:main",
        ],
    },
    packages=[
        "peepdf",
    ],
    package_data={"": ["README.md, COPYING"]},
)
